# @agentics/get

A versatile command-line tool and library for making HTTP requests, scraping web content, and automating web interactions using Node.js.

## Table of Contents

- [Features](#features)
- [Installation](#installation)
- [Usage](#usage)
  - [Command-Line Usage](#command-line-usage)
  - [Options](#options)
  - [Examples](#examples)
- [Programmatic Usage](#programmatic-usage)
- [Contributing](#contributing)
- [License](#license)

## Features

- Perform HTTP GET and POST requests.
- Scrape web pages using selectors.
- Extract text, HTML, links, images, and cookies.
- Evaluate custom JavaScript on web pages.
- Save responses and cookies to files.
- Use customizable request headers.
- Supports JavaScript evaluation with Puppeteer.

## Installation

```bash
npm install -g @agentics/get
```

## Usage

### Command-Line Usage

```bash
get [options]
```

### Options

| Option                 | Alias | Type    | Description                                                                                           | Default |
|------------------------|-------|---------|-------------------------------------------------------------------------------------------------------|---------|
| `--get`                | `-g`  | Boolean | Perform a GET request (default).                                                                      | `true`  |
| `--post`               | `-p`  | Boolean | Perform a POST request.                                                                               | `false` |
| `--url <url>`          | `-u`  | String  | URL to request. **(Required)**                                                                        |         |
| `--eval <script>`      | `-e`  | String  | Evaluate JavaScript on the page.                                                                      |         |
| `--save`               | `-s`  | Boolean | Save response and cookies to files (`response.json`, `cookies.json`).                                 | `false` |
| `--selector <css>`     | `-S`  | String  | CSS selector to scrape content from the page.                                                         |         |
| `--links`              | `-l`  | Boolean | Return all links (`<a href="">`) from the page.                                                      | `false` |
| `--text`               | `-t`  | Boolean | Return the text content of the page (default if no other output option is specified).                 | `false` |
| `--html`               | `-H`  | Boolean | Return the HTML content of the page.                                                                  | `false` |
| `--cookies`            | `-c`  | Boolean | Return cookies from the response headers.                                                             | `false` |
| `--header <key:value>` | `-h`  | String  | Custom headers for the request (can be used multiple times).                                          |         |

### Examples

#### Basic GET Request

```bash
get -u https://example.com
```

#### POST Request with Custom Headers

```bash
get -p -u https://example.com/api -h "Content-Type: application/json" -h "Authorization: Bearer token"
```

#### Scrape Text Using a CSS Selector

```bash
get -u https://example.com -S ".article-title"
```

#### Evaluate JavaScript on the Page

```bash
get -u https://example.com -e "document.title"
```

#### Get All Links from a Page

```bash
get -u https://example.com -l
```

#### Save Response and Cookies to Files

```bash
get -u https://example.com -s
```

#### Get Page HTML

```bash
get -u https://example.com -H
```

#### Get Cookies from Response Headers

```bash
get -u https://example.com -c
```

### Combining Options

You can combine multiple options to perform complex tasks. For example:

```bash
get -u https://example.com -S ".article-title" -l -c -s
```

This command will:

- Scrape content matching `.article-title`.
- Return all links from the page.
- Return cookies from the response headers.
- Save the response and cookies to files.

## Programmatic Usage

You can also use `@agentics/get` as a library in your Node.js projects.

### Importing the Module

```javascript
const get = require('@agentics/get');
```

### Using `exportFunctions`

The `exportFunctions` method allows you to perform web scraping tasks programmatically.

#### Syntax

```javascript
const results = await get(url, options);
```

#### Parameters

- `url` *(String)*: The URL to request.
- `options` *(Object, optional)*: Configuration options.

#### Available Options

- `post` *(Boolean)*: Use POST request instead of GET.
- `headers` *(Object)*: Custom headers for the request.
- `cookies` *(Boolean)*: Return cookies from the response headers.
- `links` *(Boolean)*: Return all links from the page.
- `html` *(Boolean)*: Return the HTML content of the page.
- `text` *(Boolean)*: Return the text content of the page.
- `selector` *(String)*: CSS selector to scrape content.
- `eval` *(String)*: JavaScript code to evaluate on the page.
- `save` *(Boolean)*: Save response and cookies to files.

#### Example

```javascript
const get = require('@agentics/get');

(async () => {
  try {
    const url = 'https://example.com';
    const options = {
      headers: {
        'User-Agent': 'CustomUserAgent/1.0',
      },
      text: true,
      links: true,
      cookies: true,
    };

    const results = await get(url, options);

    console.log('Text Content:', results.text);
    console.log('Links:', results.links);
    console.log('Cookies:', results.cookies);
  } catch (error) {
    console.error('Error:', error);
  }
})();
```

#### Default Options

If no options are provided, the following defaults are used:

```javascript
{
  cookies: true,
  links: true,
  html: true,
  text: true,
}
```

## Contributing

Contributions are welcome! Please open an issue or submit a pull request on [GitLab](https://gitlab.com/a4to/get).

## License

This project is licensed under the [MIT License](LICENSE).

---

**Author:** Connor Etherington
**Email:** [connor@agentics.co.za](mailto:connor@agentics.co.za)
