#!/usr/bin/env node
/**
 * @license get
 * MIT/X Consortium License
 *
 * File: app.js
 * © Agentics (Pty) Ltd <connor@agentics.co.za>
 *
*/

const axios = require('axios');
const fs = require('fs');
const yargs = require('yargs');
const cheerio = require('cheerio');
let puppeteer;

const log = (input, ...args) => {
  const colors = {
  r: '\x1b[31;1m', g: '\x1b[32;1m', b: '\x1b[34;1m', y: '\x1b[33;1m', w: '\x1b[37;1m',
  m: '\x1b[35;1m', x: '\x1b[38;2;103;208;168m', c: '\x1b[36;1m', C: '\x1b[0m' };
  input = input.concat(':C: ', ...args).toString();
  let result = input.replace(/:([rgbyxwmcC]):/g, (match, p1) => colors[p1] || '');
  console.log(result + colors.C);
};

const { JSDOM } = require('jsdom');

class Page {
  constructor(url, html) {
    this.url = url;
    this.html = html;
    this.getDoc();
  }

  async getDoc() {
    if (!this.html) {
      const response = await fetch(this.url);
      this.html = await response.text();
    }
    const dom = new JSDOM(this.html);
    this.document = dom.window.document;
    this.window = dom.window;
    return { document: this.document, window: this.window };
  }

  static async getDoc(url) {
    const response = await fetch(url);
    const html = await response.text();
    const dom = new JSDOM(html);
    return { document: dom.window.document, window: dom.window };
  }
}

class Scraper {
  constructor(html, options) {
    this.initialized = false;
    this.initPromise = this.init(html, options);
  }

  async init(html, options) {
    if (html.toString().startsWith('http')) {
      try {
        const response = await axios.get(html, options);
        this.$ = cheerio.load(response.data);
        this.response = response;
      } catch (error) {
        console.error('Error making request:', error.message);
        throw error;
      }
    } else {
      this.$ = cheerio.load(html);
    }
    this.initialized = true;
  }

  async ensureInitialized() {
    if (!this.initialized) {
      await this.initPromise;
    }
  }

  async scrape(selector) {
    await this.ensureInitialized();
    return this.$(selector).text();
  }

  async pageText() {
    await this.ensureInitialized();
    const allText = this.$('body').html();
    const removeCSSAndJS = (text) => {
      text = text.replace(/<style[^>]*>[\s\S]*?<\/style>/gi, '');
      text = text.replace(/<script[^>]*>[\s\S]*?<\/script>/gi, '');
      text = text.replace(/\s*style=["'][^"']*["']/gi, '');
      return text;
    };

    const cleanedHTML = removeCSSAndJS(allText);
    let textContent = this.$('<div>').html(cleanedHTML).text();
    textContent = textContent.replace(/(\n\s*){3,}/g, '\n\n');
    return textContent;
  }

  async evaluate(script) {
    await this.ensureInitialized();
    if (!puppeteer) {
      puppeteer = require('puppeteer-extra');
      const pluginStealth = require('puppeteer-extra-plugin-stealth');
      const userPrefs = require('puppeteer-extra-plugin-user-preferences');
      puppeteer.use(userPrefs({ userPrefs: { webkit: { webprefs: { experimentalFeatures: true } } } }));
      puppeteer.use(pluginStealth());
    }

    let executablePath = require('puppeteer').executablePath();
    if(!fs.existsSync(executablePath)) {
      log(`:y:? :x:Puppeteer executable not found. Attempting to install...`);
      try {
        await require('child_process').execSync('npx puppeteer browsers install');
        executablePath = require('puppeteer').executablePath();
      } catch (error) {
        log(`:r:- :x:Failed to install Puppeteer executable. Please install manually.`);
        process.exit(1);
      }
    }

    const launchOptions = { executablePath };

    const browser = await puppeteer.launch(launchOptions);
    const page = await browser.pages().then(pages => pages[0]);

    await page.goto(this.response.config.url, { waitUntil: 'networkidle0' });

    const result = await page.evaluate(script);
    await browser.close();

    this.evalResult = result;

    return result;
  }

  async links() {
    await this.ensureInitialized();
    return this.$('a').map((i, el) => this.$(el).attr('href')).get();
  }

  async images() {
    await this.ensureInitialized();
    return this.$('img').map((i, el) => this.$(el).attr('src')).get();
  }

  async getHtml() {
    await this.ensureInitialized();
    return this.$.html();
  }

  async getResponse() {
    await this.ensureInitialized();
    return this.response;
  }

  async cookies() {
    await this.ensureInitialized();
    return this.response.headers['set-cookie'];
  }
}

const argv = yargs
  .option('g', {
    alias: 'get',
    type: 'boolean',
    description: 'GET request (default)',
    default: true
  }).option('p', {
    alias: 'post',
    type: 'boolean',
    description: 'POST request',
    default: false
  }).option('e', {
    alias: 'eval',
    type: 'string',
    description: 'Evaluate JavaScript on the page',
  }).option('u', {
    alias: 'url',
    type: 'string',
    description: 'URL to request',
    demandOption: true
  }).option('s', {
    alias: 'save',
    type: 'boolean',
    description: 'Save response and cookies to files',
    default: false,
  }).option('S', {
    alias: 'selector',
    type: 'string',
    description: 'Selector to scrape',
  }).option('l', {
    alias: 'links',
    type: 'boolean',
    description: 'Return page links',
    default: false
  }).option('t', {
    alias: 'text',
    type: 'boolean',
    description: 'Return page text',
    default: false
  }).option('H', {
    alias: 'html',
    type: 'boolean',
    description: 'Return page HTML',
    default: false
  }).option('c', {
    alias: 'cookies',
    type: 'boolean',
    description: 'Return cookies',
    default: false
  }).option('h', {
    alias: 'header',
    type: 'array',
    description: 'Headers for the request (can be used multiple times)',
  }).argv;

const method = argv.p ? 'POST' : 'GET';
const url = argv.u;

const headers = argv.h ? argv.h.reduce((acc, header) => {
  const [key, value] = header.split(':');
  acc[key.trim()] = value.trim();
  return acc;
}, {}) : {};

const config = { method, headers, url, withCredentials: true };

const saveFiles = (response) => {
  fs.writeFileSync('response.json', JSON.stringify({
    data: response.data,
    headers: response.headers,
    cookies: response.headers['set-cookie'] || [],
  }, null, 2));

  fs.writeFileSync('cookies.json', JSON.stringify({
    cookies: response.headers['set-cookie'] || [],
  }, null, 2));

  console.log('Response and cookies saved.');
};

const scraper = new Scraper(url, config);

(async () => {
  try {
    if ((!argv.H && !argv.e && !argv.c) || argv.t) console.log(await scraper.pageText());
    if (argv.l) console.log(await scraper.links());
    if (argv.H) console.log(await scraper.getHtml());
    if (argv.S) console.log(await scraper.scrape(argv.S));
    if (argv.e) console.log(await scraper.evaluate(argv.e));
    if (argv.c) console.log(await scraper.cookies());
    if (argv.save) {
      const response = await scraper.getResponse();
      saveFiles(response);
    }
  } catch (error) {
    console.error('An error occurred:', error);
  }
})();

const exportFunctions = async (url, options = {}) => {
  if (Object.keys(options).length === 0) {
    options = {
      cookies: true,
      links: true,
      html: true,
      text: true,
    };
  }

  const scrapeOptions = {
    headers: options.headers || {},
    method: options.post ? 'POST' : 'GET',
  };

  const scraper = new Scraper(url, scrapeOptions);
  await scraper.ensureInitialized();

  const results = {};

  if (options.text) results.text = await scraper.pageText();
  if (options.links) results.links = await scraper.links();
  if (options.html) results.html = await scraper.getHtml();
  if (options.selector) results.scraped = await scraper.scrape(options.selector);
  if (options.eval) results.evaluated = await scraper.evaluate(options.eval);
  if (options.cookies) results.cookies = await scraper.cookies();

  if (options.save) {
    const response = await scraper.getResponse();
    saveFiles(response);
  }

  return results;
};

module.exports = exportFunctions;

